void init_pwm4_fast_pwm_for_axon(uint16_t prescaler, uint16_t freq_in_ms) {
  TCCR4A = 0; // diable all PWM on Timer4 while we set it up

  // set TOP so that freq is every 20ms or 50hz
  ICR4 = get_top_fast_pwm(prescaler, (uint16_t)(1000/freq_in_ms)); 

  // configure timer 4 for fast pwm mode using ICR4, with prescale of 64
  // enable pwm on non-inverting mode
  TCCR4B = _BV(WGM43) | _BV(WGM42) | _BV(CS41) | _BV(CS40);
  TCCR4A = _BV(WGM41) | _BV(COM4A1) | _BV(COM4B1) | _BV(COM4C1);

  // center servo for initialization
  OCR4A = OCR4B = OCR4C = get_duty_cycle_ratio_hs311(ICR4, SERVO_CENTER);
}

void test_servo_positions_on_E345_hs311(void) {
  uint16_t duty;
  duty = get_duty_cycle_ratio_hs311(ICR3, SERVO_MAX_LEFT);
  rprintf("setting servo to 0.75, duty: %d\r\n",duty);
  OCR3A = duty;
  OCR3B = duty;
  OCR3C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR3, 1);
  rprintf("setting servo to 1, duty: %d\r\n",duty);
  OCR3A = duty;
  OCR3B = duty;
  OCR3C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR3, 1.25);
  rprintf("setting servo to 1.25, duty: %d\r\n",duty);
  OCR3A = duty;
  OCR3B = duty;
  OCR3C = duty;
  sleep(1200);
  // 1.5 ms pulse to servo; centering the servo
  duty = get_duty_cycle_ratio_hs311(ICR3, SERVO_CENTER);
  rprintf("setting servo to 1.5 CENTER, duty: %d\r\n",duty);
  OCR3A = duty;
  OCR3B = duty;
  OCR3C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR3, 1.75);
  rprintf("setting servo to 1.75ms, duty: %d\r\n",duty);
  OCR3A = duty;
  OCR3B = duty;
  OCR3C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR3, 2);
  rprintf("setting servo to 2ms, duty: %d\r\n",duty);
  OCR3A = duty;
  OCR3B = duty;
  OCR3C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR3, 2.25);
  rprintf("setting servo to 2.25ms, duty: %d\r\n",duty);
  OCR3A = duty;
  OCR3B = duty;
  OCR3C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR3, SERVO_MAX_RIGHT);
  rprintf("setting servo to 2.5ms, duty: %d\r\n",duty);
  OCR3A = duty;
  OCR3B = duty;
  OCR3C = duty;
  sleep(1200);
}

void test_servo_positions_on_H345_hs311(void) {
  uint16_t duty;
  duty = get_duty_cycle_ratio_hs311(ICR4, SERVO_MAX_LEFT);
  rprintf("setting servo to 0.75, duty: %d\r\n",duty);
  OCR4A = duty;
  OCR4B = duty;
  OCR4C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR4, 1);
  rprintf("setting servo to 1, duty: %d\r\n",duty);
  OCR4A = duty;
  OCR4B = duty;
  OCR4C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR4, 1.25);
  rprintf("setting servo to 1.25, duty: %d\r\n",duty);
  OCR4A = duty;
  OCR4B = duty;
  OCR4C = duty;
  sleep(1200);
  // 1.5 ms pulse to servo; centering the servo
  duty = get_duty_cycle_ratio_hs311(ICR4, SERVO_CENTER);
  rprintf("setting servo to 1.5 CENTER, duty: %d\r\n",duty);
  OCR4A = duty;
  OCR4B = duty;
  OCR4C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR4, 1.75);
  rprintf("setting servo to 1.75ms, duty: %d\r\n",duty);
  OCR4A = duty;
  OCR4B = duty;
  OCR4C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR4, 2);
  rprintf("setting servo to 2ms, duty: %d\r\n",duty);
  OCR4A = duty;
  OCR4B = duty;
  OCR4C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR4, 2.25);
  rprintf("setting servo to 2.25ms, duty: %d\r\n",duty);
  OCR4A = duty;
  OCR4B = duty;
  OCR4C = duty;
  sleep(1200);
  duty = get_duty_cycle_ratio_hs311(ICR4, SERVO_MAX_RIGHT);
  rprintf("setting servo to 2.5ms, duty: %d\r\n",duty);
  OCR4A = duty;
  OCR4B = duty;
  OCR4C = duty;
  sleep(1200);
}

uint16_t get_duty_cycle_ratio_hs311(const uint16_t top_val, const float pulse_in_ms) {
  return (uint16_t)(top_val * pulse_in_ms / HS311_FREQ_MS);
}

